c========================================================================
      PROGRAM FLUX_MODEL_VBZ
c========================================================================
c
c    AUTHOR: Michael H. Denton
c    Copyright 2016 Michael H. Denton
c    Version 1.0 : 13/06/2016
c
c    GNU-GPL v3 (see below and file gpl.txt, which accompanies this code)
c
c    A summary of the first version of the model can be found in the 
c    publication that accompanies this code:
c
c    "An empirical model of electron and ion fluxes derived from
c    observations at geosynchronous orbit",  
c    M. H. Denton, M. F. Thomsen, V. K. Jordanova, M. G. Henderson,
c    J. E. Borovsky, J. S. Denton, D. Pitchford, and D. P. Hartley.
c    Space Weather, 13, 10.1002/2015SW001168, 2015.
c
c    "An improved empirical model of electron and ion fluxes at 
c    geosynchronous orbit based on upstream solar wind conditions",
c    M. H. Denton, M. G. Henderson, V. K. Jordanova, M. F. Thomsen, 
c    J. E. Borovsky, J. Woodroffe, D. P. Hartley, and D. Pitchford, 2016.
c    (under review)
c
c    The help and suggestions of co-authors is gratefully acknowledged!!!
c
c    NOTE: Flux values here are more precise than quoted in the paper 
c    due to slight improvements in the satellite centre-energies.     
cc
c    If you find the code useful, or have suggestions for changes, then 
c    please let me know.  Mick Denton - mdenton@spacescience.org
c
c=============================PROGRAM OVERVIEW===========================
c
c    The code returns flux values for GEOSYNCHRONOUS ORBIT as a function
c    of energy, local-time, and Kp index.  The underlying data for 
c    the model is from the Los Alamos National Laboratory MPA 
c    instrument (PI: M. G. Henderson).
c
c    Model bounds: ~1 eV to ~40 keV, any LT, any Kp.
c
c    CAVEAT: Electron Fluxes are only reliable at E >~100 eV
c            Below 100 eV there may be contamination by photoelectrons
c            and secondary electrons.
c
c    INPUTS:
c    The user may enter the following:
c
c      choose_en:   Energy in eV, from ~1ev to ~40000 keV.  
c                   e.g. 25500=25.5 keV.
c
c      choose_vbz:  The solar wind electric field i.e. -vBz.
c                   NOTE THE NEGATIVE IN THIS EQUATION!!!
c                   i.e. POSITIVE VALUES ARE FOR SOUTHWARD IMF!!!
c                   So you enter -[V(km/s) * Bz (nT; GSM)] 
c                   e.g. if the solar wind speed is 450 km/s and
c                        the magnetic field strength at 1 AU is 
c                        -4 nT then -V=-450 km/s and B=-4 nt so
c                        enter the value (-450)x(-4)=+1800
c
c      choose_lt:  (e.g. 12.80 LT)
c
c    OUTPUTS: 
c    The model returns the following:
c
c  Mean electron flux: (ALL F10.7) 
c  Standard deviation on the mean electron flux
c   5th percentile electron flux (five)
c  25th percentile electron flux (tfive)
c  50th percentile electron flux (med)
c  75th percentile electron flux (sfive)
c  95th percentile electron flux (nfive)
c 
c  Mean ion flux : (ALL F10.7)
c  Standard deviation on the mean ion flux
c   5th percentile ion flux (five)
c  25th percentile ion flux (tfive)
c  50th percentile ion flux (med)
c  75th percentile ion flux (sfive)
c  95th percentile ion flux (nfive)
c
c  NOTE: The typical variation with F10.7 (low to high) can be found 
c  by using the version of the model driven by Kp.
c   
c  Fluxes are given in units of (cm^-2 s^-1 sr^-1 eV^-1)
c
c    DESCRIPTION OF SUBROUTINES
c  
c    flux_model.f: 
c          The MAIN subroutine.
c    bilinear.f: 
c          Function which performs bilinear interpolation.
c    calc.f:
c          Subroutine which takes the input and returns flux values.
c    data_read.f:
c          Subroutine to read in the underlying MPA data files.
c
c    GNU GENERAL PUBLIC LICENSE
c    This program is free software: you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation, either version 3 of the License, or
c    (at your option) any later version.
c
c    This program is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details (file gpl.txt).
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>
c
c    Please feel free to modify the code to suit your own needs in line 
c    with this license.
c
c========================================================================

      IMPLICIT NONE

c     DEFINE INTEGER VARIABLES
      INTEGER(KIND=8) VBZS, ENS, LTS, VALS
      INTEGER(KIND=8) I, J, K
      INTEGER(KIND=8) ARR_SIZE

c     DEFINE PARAMETERS
      PARAMETER(VBZS=33, ENS=40, LTS=24, VALS=7)
c     ARR_SIZE IS SET TO MATCH INPUT FILE LENGTH
      PARAMETER(ARR_SIZE=40)

c     DEFINE CHARACTER ARRAYS
      CHARACTER head_arr(1)*16
      CHARACTER str_arr(VALS)*10
      CHARACTER*100 model_input_file

c     DEFINE REAL ARRAYS
      REAL(KIND=8) mean_e_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) std_e_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) five_e_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) tfive_e_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) med_e_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) sfive_e_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) nfive_e_flux_all(VBZS,ENS,LTS)

      REAL(KIND=8) mean_i_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) std_i_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) five_i_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) tfive_i_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) med_i_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) sfive_i_flux_all(VBZS,ENS,LTS)
      REAL(KIND=8) nfive_i_flux_all(VBZS,ENS,LTS)

      REAL(KIND=8) sc_pot_all(VBZS,LTS)

      REAL(KIND=8) final_e_flux_all(VALS),  final_i_flux_all(VALS)

      REAL(KIND=8) final_sc_pot_all

      REAL(KIND=8) input_arr(3,ARR_SIZE)

c     DEFINE REAL VARIABLES
      REAL(KIND=8) choose_vbz, choose_en, choose_lt

      DATA str_arr/
     >'      MEAN','   STD-DEV','   5TH PER','  25th PER','   MEDIAN',
     >'  75th PER','  95th PER'/

      DATA head_arr/
     >'       ALL F10.7'/

c     ===================================================================
c     --------------END DEFINITIONS-BEGIN MODEL--------------------------
c     ===================================================================

c     PRINT HEADER TO SCREEN
      WRITE(6,*) ''
      WRITE(6,*) '====================================================='
      WRITE(6,*) '   AUTHOR:   Michael H. Denton'
      WRITE(6,*) '   CODE:     FLUX_MODEL_VBZ'
      WRITE(6,*) '   VERSION:  1.0 (13/06/2016)'
      WRITE(6,*) ''
      WRITE(6,*) '  flux_model_vbz.f'
      WRITE(6,*) '        bilinear.f'
      WRITE(6,*) '            calc.f'
      WRITE(6,*) '       data_read.f'
      WRITE(6,*) ''
      WRITE(6,*) '  This is free software under the terms of the GNU '
      WRITE(6,*) '  General Public License v3.  You are welcome to '
      WRITE(6,*) '  redistribute it under certain conditions. You'
      WRITE(6,*) '  should have received a copy of the GNU General'
      WRITE(6,*) '  Public License along with this program. If not, see'
      WRITE(6,*) '  http://gnu.org/licenses'
      WRITE(6,*) ''
      WRITE(6,*) '  Electron energy channels below 100 eV are likely '
      WRITE(6,*) '  by photoelectrons and secondaries!     ' 
      WRITE(6,*) ''
      WRITE(6,*) '  Feel free to modify in accordance with the license!'
      WRITE(6,*) ''
      WRITE(6,*) '  This program comes with ABSOLUTELY NO WARRANTY!!!'
      WRITE(6,*) '====================================================='
      WRITE(6,*) ''

c      CALL SLEEP(2)

c     NOW READ IN THE DATA!!!
      CALL DATA_READ(
     >  mean_e_flux_all,  std_e_flux_all,  five_e_flux_all,
     > tfive_e_flux_all,  med_e_flux_all, sfive_e_flux_all,
     > nfive_e_flux_all,
     >  mean_i_flux_all,  std_i_flux_all,  five_i_flux_all,
     > tfive_i_flux_all,  med_i_flux_all, sfive_i_flux_all,
     > nfive_i_flux_all,  sc_pot_all)

c     THIS IS WHERE YOU CAN SET YOUR OWN INPUTS...DON'T SCREW UP VBZ!
c     SEE THE NOTE ON LINES 43-50 ABOVE

c     READ IN LIST OF INPUTS FROM FILE
      model_input_file='model_input.txt'
      OPEN(UNIT=1, FILE=model_input_file, STATUS='OLD')
      DO 5 I=1,ARR_SIZE
        READ(1,*) (input_arr(J,I),J=1,3)
c        READ(1,200) (input_arr(J,I),J=1,3)
c        PRINT*, (input_arr(j,I), J=1,3)
 5    CONTINUE
      CLOSE(UNIT=1)

c     LOOP AROUND THE CALL_CALC ROUTINE WITH YOUR OWN INPUTS AND DUMP
c     YOUR OWN OUTPUTS IF REQUIRED
c      choose_vbz=-2000.
c      choose_en=5280.D0
c      choose_lt=17.0D0

      PRINT*, '-VBZ=', input_arr(1,1)
      PRINT*, 'LT=',   input_arr(3,1)
      WRITE(6,20) head_arr(1)

      DO 6 I=1,ARR_SIZE
        choose_vbz=input_arr(1,I)
        choose_en =input_arr(2,I)
        choose_lt =input_arr(3,I)

c     PERFORM THE INTERPOLATION AND RETURN THE VALUES
        CALL CALC(choose_vbz, choose_en, choose_lt,
     >   mean_e_flux_all,   std_e_flux_all,   five_e_flux_all,
     >   tfive_e_flux_all,  med_e_flux_all,  sfive_e_flux_all,
     >   nfive_e_flux_all,
     >   mean_i_flux_all,   std_i_flux_all,   five_i_flux_all,
     >   tfive_i_flux_all,  med_i_flux_all,  sfive_i_flux_all,
     >   nfive_i_flux_all,  sc_pot_all, 
     >   final_e_flux_all,  final_i_flux_all, final_sc_pot_all)

c      WRITE(6,*) ''
c      WRITE(6,*) '===================================================='
c      WRITE(6,30)'   CHOSEN -VBZ =',str_vbz(choose_vbz),'(INDEX=', 
c     > vbz_array(choose_vbz),')'
c      WRITE(6,40)'CHOSEN ENERGY =',choose_en, ' keV'
c      WRITE(6,50)'    CHOSEN LT =',choose_lt, ' LT'
c      WRITE(6,*) '===================================================='
c      WRITE(6,*) ''

c     PRINT OUT ELECTRON VALUES TO SCREEN
c      WRITE(6,*) '*** ELECTRONS ***'
c      WRITE(6,20) head_arr(1)
c      WRITE(6,10) str_arr(1), final_e_flux_all(1)
c      WRITE(6,10) str_arr(2), final_e_flux_all(2)
c      WRITE(6,10) str_arr(3), final_e_flux_all(3)
       WRITE(6,11) choose_en, final_e_flux_all(5), final_i_flux_all(5)
c      WRITE(6,10) str_arr(6), final_e_flux_all(6)
c      WRITE(6,10) str_arr(7), final_e_flux_all(7)
c      WRITE(6,*) ''

c     PRINT OUT ION VALUES
c      WRITE(6,*) '*** IONS ***'
c      WRITE(6,20) head_arr(1)
c      WRITE(6,10) str_arr(1), final_i_flux_all(1)
c      WRITE(6,10) str_arr(2), final_i_flux_all(2)
c      WRITE(6,10) str_arr(3), final_i_flux_all(3)
c      WRITE(6,10) str_arr(4), final_i_flux_all(4)
c        WRITE(6,11) choose_en, final_i_flux_all(5)
c      WRITE(6,10) str_arr(6), final_i_flux_all(6)
c      WRITE(6,10) str_arr(7), final_i_flux_all(7)
  
c        WRITE(6,12) choose_vbz, final_sc_pot_all

 6    CONTINUE

      PRINT*, 'SC POTENTIAL=', final_sc_pot_all

c     PROGRAM COMPLETE
      WRITE(6,*) ''
      WRITE(6,*) '-------------FLUX MODEL COMPLETE-------------'
      WRITE(6,*) ''

c 10   FORMAT(A16, 4F18.6)
 11   FORMAT(5F18.6)
 12   FORMAT(2F18.6)

 20   FORMAT(17x,  4A18)
 30   FORMAT(A15,A3,2x,A7,I2,A1)
 40   FORMAT(A15,F10.3,A4)
 50   FORMAT(A15,F6.2,A3)
 200  FORMAT(3D18.6)

      STOP
 
      END
c=======================END FLUX MODEL===================================


